/*
	DynAPI Distribution
	DynEvent, EventObject, DynElement Classes

	The DynAPI Distribution is distributed under the terms of the GNU LGPL license.
*/

function DynEvent(type,src) {
	this.inherit('DynObject');
	this.type = type;
	this.src = src;
	this.origin = src;
	this.propagate = true;
	this.bubble = false;
	this.bubbleChild = null;
	this.defaultValue = true;
};
var p = dynapi.setPrototype('DynEvent','DynObject');
p.getType = function() {return this.type};
p.getSource = function() {return this.src};
p.getOrigin=function() {return this.origin};
p.stopPropagation = function() {this.propagate = false};
p.preventBubble = function() {this.bubble = false};
p.preventDefault = function() {this.defaultValue = false};
p.getBubbleChild = function() {return this.bubbleChild};

function EventObject() {
	this.inherit('DynObject');
	this._listeners = [];
}
p = dynapi.setPrototype('EventObject','DynObject');
p._addEventListener = DynAPI.functions.Null;
p.addEventListener = function(el) {
	if (el) {
		for (var i=0;i<this._listeners.length;i++) if (this._listeners[i]==el) return;
		this._listeners[this._listeners.length] = el;
	
		if (this._listeners.length && this.captureMouseEvents) this.captureMouseEvents();
	}
};
p._removeEventListener = DynAPI.functions.Null;
p.removeEventListener = function(el) {
	if (el) {
		DynAPI.functions.removeFromArray(this._listeners, el, false);
		if (!this._listeners.length && this.releaseMouseEvents) this.releaseMouseEvents();
	}
};
p.removeAllEventListeners = function() {
	this._listeners = [];
};
p.invokeEvent = function(type,e,args) {
	if (!e) e = new DynEvent(type,this);
	if (this._listeners.length) for (var i=0;i<this._listeners.length;i++) {
		if (this._listeners[i]["on"+type]) this._listeners[i]["on"+type](e,args);
		if (!e.propagate) break;
	}
	if (this["on"+type]) this["on"+type](e,args);
	if(e.bubble && this.parent) {
		e.x += e.src.x;
		e.y += e.src.y;
		e.src = this.parent;
		e.bubbleChild = this;
		this.parent.invokeEvent(type,e,args);
	}
};

function DynElement() {
	this.inherit('EventObject');
	this.isChild = false;
	this._created = false;
	this.parent = null;
	this._dyndoc = null;
	this.children = [];
	this._childAnchors = [];
};
p = dynapi.setPrototype('DynElement','EventObject');
p.addChild = function(c) {
	if (!c) return DynAPI.debug.print("Error: no object sent to [DynLayer].addChild()");
	if (c.isChild) c.removeFromParent();
	c.isChild = true;
	c.parent = this;
	if (c._saveAnchor) {
		c.setAnchor(c._saveAnchor);
		delete c._saveAnchor;
	}
	if (this._created) c._create();
	this.children[this.children.length] = c;
	return c;
};
p.removeChild = function(c) {
	var l = this.children.length;
	for (var i=0;i<l && this.children[i]!=c;i++);
	if (i!=l) {
		c._remove();
		c._created = false;
		c.isChild = false;
		c.parent = null;
		c.dyndoc = null;
		this.children[i] = this.children[l-1];
		this.children[l-1] = null;
		this.children.length--;
	}
};
p.deleteChild = function(c) {
	c.removeFromParent();
	c._delete();
};
p.deleteAllChildren = function() {
	var l = this.children.length;
	for(var i=0;i<l;i++) {
		this.children[i].del();
		delete this.children[i];
	}
	this.children = [];
};
p.deleteFromParent = function () {
	if (this.parent) this.parent.deleteChild(this);
};
p.removeFromParent = function () {
	if (this.parent) this.parent.removeChild(this);
};
p._create = p._remove = p._delete = p._destroy = DynAPI.functions.Null;
p.getAll = function() {
	var ret = [];
	var temp;
	var l = this.children.length;
	for(var i=0;i<l;i++) {
		ret[this.children[i].id] = this.children[i];
		temp = this.children[i].getAll();
		for(var j in temp) ret[j] = temp[j];
	}
	return ret
};
p.isParentOf = function(obj,equality) {
	if (!obj) return false;
	return (equality && this==obj) || this.getAll()[obj.id]==obj;
};
p.isChildOf = function(obj,equality) {
	if(!obj) return false;
	return (equality && this==obj) || obj.getAll()[this.id]==this;
};
DynElement._flagEvent = function(obj,type) {
	if (type=="create") obj._created = true;
	obj.invokeEvent(type);
	var l=obj.children.length;
	for (var i=0; i<l; i++) {
		DynElement._flagEvent(obj.children[i],type);
	}
};
p.updateAnchor = function() {
	this.parent._updateAnchor(this.id);
};
p._updateAnchor = function(id) {
	var dlyr = DynAPI.document.all[id];
	var a = this._childAnchors[id];
	var tw = this.w;
	var th = this.h;
	if (tw==null && th==null) return;
	var x = a.left;
	var y = a.top;
	var w = null;
	var h = null;
	if (a.centerH!=null) {
		x = Math.ceil(tw/2 - dlyr.getWidth()/2 + a.centerH);
	}
	else if (a.right!=null) {
		if (a.left!=null) w = tw - a.right - a.left;
		else x = tw - dlyr.getWidth() - a.right;
	}
	if (a.centerV!=null) {
		y = Math.ceil(th/2 - dlyr.getHeight()/2 + a.centerV);
	}
	else if (a.bottom!=null) {
		if (a.top!=null) h = th - a.bottom - a.top;
		else y = th - dlyr.getHeight() - a.bottom;
	}
	dlyr.setLocation(x,y);
	dlyr.setSize(w,h);
};
p._updateAnchors = function() {
	var tw = this.w;
	var th = this.h;
	if (tw==null && th==null) return;
	for (id in this._childAnchors) this._updateAnchor(id);
};