/*
	DynAPI Distribution
	Debugger

	The DynAPI Distribution is distributed under the terms of the GNU LGPL license.
*/

function Debugger() {
	this.inherit('DynObject');
	this.win = null;
}
dynapi.setPrototype('Debugger','DynObject');
Debugger.prototype.isLoaded = function() {return (this.win && this.win.document.debugform)};
Debugger.prototype.status = function(str) {
	if (this.isLoaded()) {
		for (var i=1;i<arguments.length;i++) {
			str += ', '+arguments[i];
		}
		this.win.document.debugform.stat.value = str;
	};
};
Debugger.prototype.inspect = function(obj,showFunctions) {
	this.print('Inspecting:');
	var v;
	for (var i in obj) {
		if (obj[i]==null) v = "null";
		else if (typeof(obj[i])=='function') {
			if (showFunctions==false) continue;
			else v = '[Function]';
		}
		else if (typeof(obj[i])=='object' && obj[i].length) v = 'Array ['+obj[i]+']';
		else if (typeof(obj[i])=='object') v = '[Object]';
		else v = obj[i];
		this.print('    '+i+' = '+v);
	}
};
Debugger.prototype.print = function(s) {
	if (this.isLoaded()) {
		for (var i=1;i<arguments.length;i++) {  // accommodate print(v1,v2,v3)
			s += ', '+arguments[i];
		}
		this.win.document.debugform.print.value += s +'\n';
	}
	else {
		if (!dynapi._debugBuffer) dynapi._debugBuffer = s;
		else dynapi._debugBuffer += '\n'+s;
	}
};
Debugger.prototype.error = function(msg, url, lno) {
	if (url && url.indexOf(dynapi.documentPath)==0) {
		url = url.substring(dynapi.documentPath.length);
	}
	this.print('Error:'+ (lno? ' Line '+lno : '') +' ['+url+']\n       '+msg);
};
Debugger.prototype.evaluate = function(str) {
	window.eval(str);
};
Debugger.prototype.setEvaluate = function(str) {
	if (this.isLoaded()) this.win.document.debugform.eval.value = str
};
Debugger.prototype.open = function() {
	var p = dynapi.library.path;
	if (!this.isLoaded() && p) {
		var url = dynapi.documentPath+p+'debug.html#';
		var w = dynapi.ua.mac? (dynapi.ua.ie?330:300) : 360;
		var h = dynapi.ua.mac? (dynapi.ua.ie?405:365) : dynapi.ua.ie? 420:465;
		this.win = window.open(url,'debugwin','width='+w+',height='+h+',scrollbars=no,status=no,toolbar=no,resizable=no');
		/*if (dynapi.ua.ns4) {  // move the debug window to the right edge?
			var f = dynapi.frame;
			var ns4 = dynapi.ua.ns4;
			var sX = ns4? f.screenX : f.screenLeft;
			var sY = ns4? f.screenY : f.screenTop;
			var sW = ns4? f.outerWidth : 100;
			if (sX!=null && sY!=null && sW!=null) this.win.moveTo(sX+sW,sY);
		}*/
		this.win.focus();
		if (dynapi._debugBuffer) {
			dynapi.debug.print(dynapi._debugBuffer);
			dynapi._debugBuffer = '';
		}
		dynapi.frame.onerror = function(msg, url, lno) {
			dynapi.debug.error(msg, url, lno);
		};
	}
};
Debugger.prototype.close = function() {
	if (this.isLoaded()) {
		this.win.close();
		this.win = null;
	}
};
dynapi.debug = new Debugger();
dynapi.debug.open();

/*dynapi.onUnload(function() {
	dynapi.debug.close();  // close the debug window on unload
});*/
